# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib import messages
from django.core.cache import cache
from django.http import (
    HttpResponseServerError,
    HttpResponse,
    Http404,
    HttpResponseRedirect,
)
from django.shortcuts import get_object_or_404, render, redirect
from django.template.loader import render_to_string
from django.utils.dateformat import format
from django.utils.safestring import mark_safe
from django.urls import reverse

from .frontend import (
    get_day_events_for,
    get_timetables_for,
    get_week_info,
    get_next_event_for_user,
    get_next_event_for_edt_ade_group,
    InvalidTimetablesException,
)
from .ics_generator import ics_response
from .models import EdtAdeGroup, EdtRecurring
from .timetable import Category
from .viewer import EventsViewer
from .forms import RecurringForm
from lib.access_auth import access_restricted, none_if_anonymous
from lib.tools import confirmation, erreur
from portailuser.models import Ecole, PortailUser

from collections import OrderedDict
from datetime import date, datetime, timedelta
import logging


EDT_WIDGET_CACHE_NAME = "edt_widget_{}"
LOGGER = logging.getLogger(__name__)
WIDGET_NAMES = {
    Category.ADMIN: "de formalités administratives",
    Category.COURS: "de cours",
    Category.SPORT: "de sport",
    Category.TD: "de TD",
    Category.TP: "de TP",
    Category.PROJET: "de projet",
    Category.EXAMEN: "d’examen",
    Category.CLUB: "de club",
}


@access_restricted
def edt(request, offset_text=None):
    if not request.user.has_configured_timetables:
        return redirect("edt:widget_settings")

    if offset_text:
        offset = int(offset_text)
    else:
        offset = 0

    if offset == 0 and offset_text:
        return redirect("edt:edt", permanent=True)

    try:
        evt_lists, has_prev, has_next = get_week_info(request.user, offset)
    except InvalidTimetablesException:
        return erreur(
            request,
            "L’emploi du temps est temporairement " "indisponible.",
            "Emploi du temps",
        )

    if evt_lists is None:
        raise Http404("Out of bounds offset")

    first_day = date.today()
    first_day += timedelta(days=7 * offset - first_day.weekday())

    url_prefixe = "webcal://" + settings.URLS_SITES[request.ecole.id - 1]

    c = {
        "week": first_day.isocalendar()[1],
        "offset": offset,
        "first_day": first_day,
        "last_day": first_day + timedelta(days=6),
        "events_viewer": EventsViewer(evt_lists, first_day),
        "prev": "{:+d}".format(offset - 1) if has_prev else None,
        "next": "{:+d}".format(offset + 1) if has_next else None,
        "url_prefixe": url_prefixe,
    }

    return render(request, "edt/edt.html", c)


def edt_ics_user(request, username):
    user = PortailUser.objects.get_from_ldap_or_404(username)

    if not user.has_configured_timetables:
        raise Http404("Emploi du temps non configuré")

    try:
        timetables = get_timetables_for(user)
    except InvalidTimetablesException as exc:
        return HttpResponseServerError(str(exc))

    return ics_response(
        request,
        timetables,
        f"user:{user.username}",
        f"Emploi du temps de {user.first_name}",
    )


@none_if_anonymous
def widget(request):
    if not request.user.has_configured_timetables:
        return "<dd>Ce module n’est pas configuré.</dd>"

    cache_name = EDT_WIDGET_CACHE_NAME.format(request.user.username)
    result = cache.get(cache_name)

    if result == "":
        return None

    if result is not None:
        return result

    disp_tomorrow = datetime.now().hour >= 18
    offset = 1 if disp_tomorrow else 0

    try:
        events = get_day_events_for(request.user, day_offset=offset, ignore_conflicting=True)
    except InvalidTimetablesException as exc:
        LOGGER.warning(exc)
        return "<dd>Erreur de récupération des données !</dd>"

    durations = OrderedDict()

    for event in events:
        evt_type = event.category
        new_duration = durations.get(evt_type, 0) + int(round(event.duration))
        durations[evt_type] = new_duration

    cur_time = datetime.now()
    cur_time = (cur_time.hour * 60 + cur_time.minute) * 60 + cur_time.second
    if disp_tomorrow:  # Change at midnight
        timeout = 86400 - cur_time
    else:  # Change at 18:00:00 this day
        timeout = 64800 - cur_time  # 18 * 60 * 60

    events = []
    for event, duration in durations.items():
        fragment = WIDGET_NAMES.get(event)
        if fragment:
            events.append((duration, fragment))

    if not events:
        cache.set(cache_name, "", timeout)
        return None

    semaine_suivante = datetime.now().weekday() == 6 and datetime.now().hour >= 18

    c = {
        "tomorrow": disp_tomorrow,
        "events": events,
        "end_week": semaine_suivante,
    }

    result = render_to_string("edt/widget.html", c)

    cache.set(cache_name, result, timeout)

    return result


@access_restricted
def widget_settings(request):
    user = request.user
    error = False
    selected_groups = dict()
    ecoles = Ecole.objects.all()

    if request.method == "POST" and "settings_submit" in request.POST:
        club_selected = request.POST.get("edt_club_events", -1)
        try:
            club_selected = int(club_selected)
        except ValueError:
            club_selected = -1

        if club_selected < 0 or club_selected > 2:
            error = True

        else:
            db_groups = []
            for ecole in ecoles:
                school_groups = f"edt_groups_{ecole.ident}"
                selected_groups[ecole.ident] = set(request.POST.getlist(school_groups))
                db_groups_ = EdtAdeGroup.objects.filter(gid__in=selected_groups[ecole.ident], ecole=ecole)
                db_groups.extend(db_groups_)
            user.edt_club_events = club_selected

            user.edt_ade_groups.clear()
            if db_groups:
                user.edt_ade_groups.add(*db_groups)

            user.save()
            cache.delete(EDT_WIDGET_CACHE_NAME.format(user.username))

            return redirect("edt:edt")
    else:
        for ecole in ecoles:
            selected_groups[ecole.ident] = set(str(group.gid) for group in user.edt_ade_groups.filter(ecole=ecole))
        club_selected = user.edt_club_events

    if request.ecole == Ecole.inp:
        ade_groups = EdtAdeGroup.objects.exclude(gid=None).order_by("ecole", "name")
    else:
        ade_groups = EdtAdeGroup.objects.exclude(gid=None).filter(ecole=request.ecole).order_by("name")
    ecoles = []
    groups = None
    ecole_act = None

    for group in ade_groups:
        if ecole_act != group.ecole:
            ecole_act = group.ecole
            groups = []
            ecoles.append((ecole_act, groups))

        if ecole_act.ident in selected_groups:
            checked = str(group.gid) in selected_groups[ecole_act.ident]
        else:
            checked = False
        groups.append((group, checked))
        groups.sort(key=lambda group: group[1], reverse=True)

    c = {
        "error": error,
        "ecoles": ecoles,
    }

    if club_selected is not None:
        c[f"edt_club_events_select_{club_selected}"] = mark_safe(' checked="checked"')

    return render(request, "edt/settings.html", c)


def next(request, ident):
    request.desactiver_widgets = True

    try:
        offset = min(0, int(request.GET.get("offset", "0")))
    except ValueError:
        offset = 0

    error = None

    if ident.isdigit() or ident[0] == "g" and ident[1:].isdigit():
        if ident[0] == "g":
            ident = ident[1:]
        # TODO: get school
        school = Ecole.n7

        try:
            next_event = get_next_event_for_edt_ade_group(int(ident), school, offset)
        except KeyError:
            error = "Groupe inconnu !"
        except InvalidTimetablesException:
            error = "Les données sont invalides !"
    else:
        try:
            user = PortailUser.objects.get_from_ldap(ident)
        except PortailUser.DoesNotExist:
            user = None
            error = "Utilisateur inconnu !"

        if user:
            if user.has_configured_timetables:
                try:
                    next_event = get_next_event_for_user(user, offset)
                except InvalidTimetablesException:
                    error = "Les données sont invalides !"
            else:
                error = (
                    f"""Tu n’as pas configuré l’emploi du temps, {user.get_nickname()}.\n"""
                    """Il faut aller sur """
                    """http://www.bde.enseeiht.fr/services/edt/reglages"""
                )

    if error:
        return HttpResponse(error, content_type="text/plain;charset=utf-8")

    if next_event is None:
        return HttpResponse("Rien de prévu…")

    response = HttpResponse(content_type="text/plain;charset=utf-8")

    response.write(format(next_event.start, "l j F, H:i"))
    response.write(" ➜ ")
    if next_event.start.date() == next_event.end.date():
        response.write(format(next_event.end, "H:i"))
    else:
        response.write(format(next_event.end, "l j F, H:i"))
    response.write(" : ")

    if next_event.type:
        response.write(f"{next_event.type} – ")

    response.write(next_event.title)

    if next_event.locations:
        response.write(" en " + ", ".join(next_event.locations))

    if next_event.people:
        response.write(" avec " + ", ".join(next_event.people))

    response.write(".")

    return response


@access_restricted
def recurring(request, event_id=None):
    if event_id is not None:
        event = get_object_or_404(EdtRecurring, pk=event_id)
        event_mod = True
    else:
        event = None
        event_mod = False

    if request.method == "POST":
        form = RecurringForm(request.POST)
        if form.is_valid():
            event = form.save()
            etu = request.user
            if event_mod:
                etu.edtrecurring.get(pk=event_id).delete()
                messages.success(request, "Événement modifié avec succès.")
            else:
                messages.success(request, "Événement créé avec succès.")
            etu.edtrecurring.add(event)

            return HttpResponseRedirect(reverse("edt:recurring"))
    else:
        form = RecurringForm(instance=event)

    events = request.user.edtrecurring.all()

    c = {
        "form": form,
        "events": events,
    }

    return render(request, "edt/recurring.html", c)


@access_restricted
def del_recurring(request, event_id):
    event = get_object_or_404(EdtRecurring, pk=event_id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            event.delete()
            cache.clear()
            messages.success(request, "Évènement supprimé avec succès.")
        return redirect("edt:recurring")

    message = """Êtes-vous sûr(e) de vouloir supprimer cet événement ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer un événement")
